#######################################################################
# Testing CVX from R (ran with R 3.1.2 on a Macbook with OS X 10.9.5) #
#######################################################################
# Requirements:
# 1. Matlab installed
# 2. CVX downloaded
# 3. If you are using a version of MATLAB dowloaded from CMU, make sure your VPN client
# is on and you are logged into the CMU campus network

# CVX setup for R
library( CVXfromR )
###################################################################
# MAKE SURE THAT YOU CHANGE THE FOLLOWING TWO PATHS APPROPRIATELY #
###################################################################
setup.directory = '/Users/mattiaciollaro/Documents/MATLAB/cvx' # this is where cvx_setup.m is located
path.to.matlab = '/Applications/MATLAB_R2014b.app/bin/matlab' # this is where the matlab script is located

# lasso test from website (http://faculty.bscb.cornell.edu/~bien/cvxfromr.html)
n = 50
p = 10
x = matrix( rnorm( n*p ), n, p )
beta = rnorm( p )
y = x%*%beta + 0.1 * rnorm( n )

cvx.code.web = paste( 'variables b(p)', 'minimize( square_pos( norm( y - x*b, 2 ) ) / 2 + lam * norm( b, 1 ) )', sep=';' )
lasso = CallCVX( cvx.code.web, const.vars = list( p=p, y=y, x=x ,lam=2 ), opt.var.names='b', setup.dir=setup.directory, matlab.call=path.to.matlab ) # on Windows machines, the default value of matlab.call='matlab' should work and you probably don't need to specify the path to the matlab script

lasso

##########################################################################################################
# TRY TO RUN THE ABOVE CODE AND CHECK THAT CVXfromR IS RUNNING AND PRODUCING AN OUTPUT SIMILAR TO 
#
# $b
#  [1] -0.22281429  0.48750498 -0.65946443 -1.09431242 -0.26432753 -1.88884925  1.03309715 -0.09575568
#  [9]  0.42257040 -0.37231911
#
# $cvx_optval
#       V1 
# 13.94465 
#
# $time
#       V1 
# 2.16349 
##########################################################################################################

# least squares test to double-check that everything works properly
X = matrix( rnorm( 100 ), nrow=25, ncol=4 )
b = c( 1, 2, 0 ,3 )
y = X %*% b

cvx.code = paste( 'variables b( 4 )', 'minimize sum_square( y - X*b )', sep=';' )
least.squares = CallCVX( cvx.code, const.vars=list( y=y, X=X ), opt.var.names='b', setup.dir=setup.directory, matlab.call=path.to.matlab )

b
least.squares$b


####################################################################################################################
# SVM TEST #
############
##############################################################
# MAKE SURE THAT YOU CHANGE THE FOLLOWING PATH APPROPRIATELY #
##############################################################
load( '/Users/mattiaciollaro/Desktop/HW1_Q4/SVM_data.RData' )

# primal formulation
n = length( SVMdata$y )
p = dim( SVMdata$X )[2]
C = SVMdata$Clist[length( SVMdata$Clist )]

###################################################################
cvx.code.svm = # YOUR CVXfromR CODE HERE

SVM = # YOUR CALL TO THE CallCVX FUNCTION HERE
###################################################################

# dual formulation
X = SVMdata$X
y = SVMdata$y

###################################################################
cvx.code.svm.dual = # YOUR CVXfromR CODE HERE

SVM.dual = # YOUR CALL TO THE CallCVX FUNCTION HERE
###################################################################

# compare optimal objective function values for primal and dual formulations
crit.opt = SVM$cvx_optval
crit.opt.dual = SVM.dual$cvx_optval

crit.opt
crit.opt.dual

# compare beta from the solution of primal formulation and dual formulation
b0 = SVM$b0
b.primal = SVM$b
b.dual = t( X ) %*% ( SVM.dual$a * y)

b.primal
b.dual

# plot the separating hyperplane
x1 = seq( min( X[,1] ), max( X[,1] ), length = 100 )
x2 = -b0 / b.primal[2] - b.primal[1] / b.primal[2] * x1

plot( x1, x2, type='l', main='Separating hyperplane', xlab=expression( x[1] ), ylab=expression( x[2] ) )
abline( h=0, lwd=0.5 )
abline( v=0, lwd=0.5 )

# misclassification rate
# 1. fit the model for all the values of C
Cvalues = as.numeric( SVMdata$Clist )

###################################################################
SVM = # YOUR CALL TO THE CallCVX.varyparam FUNCTION HERE
###################################################################

# function to get predicted labels for a given set of parameters b0, b, and for a given matrix X of test data
predict.SVM = function( b0, b, X.test ){
	
	yhat01 = as.numeric( X.test %*% b + b0 >= 0 )
	yhat = 2 * yhat01 - 1
	
	return( yhat)
	
}

# create matrix with predicted labels for each value of C (arranged in columns)
X.test = SVMdata$Xtest
y.test = SVMdata$ytest
predictions = matrix( NA, nrow=length( y.test ), ncol=length( SVMdata$Clist ))

for( i in 1:length( SVMdata$Clist ) ){
	
	predictions[,i] = predict.SVM( SVM$b0[[i]], SVM$b[[i]], X.test )
	
}

# compute misclassification error
misclassification.matrix = predictions - y.test%*%t( rep( 1, length( SVMdata$Clist ) ) )
misclassification.matrix[misclassification.matrix != 0] = 1
misclassification.error = apply( misclassification.matrix, 2, mean )

# plot misclassification error
plot( log(SVMdata$Clist,10), misclassification.error, type='o', main='Misclassification error', xlab=expression( log[10]( C ) ), ylab='misclassification rate' )